<?php

namespace Pinpoint\SalesRep\Block\Adminhtml\Order\View\Tab;

use Magento\Backend\Block\Template;
use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Tab\TabInterface;
use Magento\Framework\Phrase;
use Magento\Framework\Registry;
use Magento\Sales\Model\Order;
use Magento\User\Model\ResourceModel\User\CollectionFactory;
use Pinpoint\SalesRep\Model\ResourceModel\SalesRepsList\CollectionFactory as SalesRepCollectionFactory;

/**
 * Class SalesRep
 */
class SalesRep extends Template implements TabInterface
{
    /**
     * @var integer
     */
    const DEFAULT_SALESREP_ID = 0;

    /**
     * @var string
     */
    const DEFAULT_SALESREP_NAME = '';
    const DEFAULT_TAB_CLASS = 'ajax only';

    /**
     * @var string
     */
    protected $_template = 'order/view/tab/salesrep.phtml';

    /**
     * @var CollectionFactory
     */
    protected $userCollectionFactory;

    /**
     * @var SalesRepCollectionFactory
     */
    protected $salesRepCollectionFactory;

    /**
     * View constructor.
     * @param Context $context
     * @param Registry $registry
     * @param CollectionFactory $userCollectionFactory
     * @param SalesRepCollectionFactory $salesRepCollectionFactory
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        CollectionFactory $userCollectionFactory,
        SalesRepCollectionFactory $salesRepCollectionFactory,
        array $data = []
    ) {
        $this->userCollectionFactory = $userCollectionFactory;
        $this->_coreRegistry = $registry;
        $this->salesRepCollectionFactory = $salesRepCollectionFactory;
        parent::__construct($context, $data);
    }

    /**
     * Retrieve order model instance
     *
     * @return \Magento\Sales\Model\Order
     */
    public function getOrder()
    {
        return $this->_coreRegistry->registry('current_order');
    }
    /**
     * Retrieve order model instance
     *
     * @return \Magento\Sales\Model\Order
     */
    public function getOrderId(): Order
    {
        return $this->getOrder()->getEntityId();
    }

    /**
     * Retrieve order increment id
     *
     * @return string
     */
    public function getOrderIncrementId(): string
    {
        return $this->getOrder()->getIncrementId();
    }
    /**
     * Return tab label
     * @return Phrase
     * {@inheritdoc}
     */
    public function getTabLabel(): Phrase
    {
        return __('Sales Representative');
    }

    /**
     * Return tab title
     * @return Phrase
     * {@inheritdoc}
     */
    public function getTabTitle(): Phrase
    {
        return __('Sales Representative');
    }

    /**
     * Return can show tab
     * @return bool
     * {@inheritdoc}
     */
    public function canShowTab(): bool
    {
        return true;
    }

    /**
     * Return is hidden
     * @return bool
     * {@inheritdoc}
     */
    public function isHidden(): bool
    {
        return false;
    }

    /**
     * Get cache lifetime
     * @return null
     * {@inheritdoc}
     */
    public function getCacheLifetime()
    {
        return null;
    }

    /**
     * Get tab Class
     *
     * @return string
     */
    public function getClass(): string
    {
        return self::DEFAULT_TAB_CLASS;
    }

    /**
     * Get Tab Url
     *
     * @return string
     */
    public function getTabUrl(): string
    {
        // customtab is a adminhtml router we're about to define
        // the full route can really be whatever you want
        return $this->getUrl('salesrep/showsalesrep/index', ['_current' => true]);
    }

    /**
     * Retrieve sales representative user ID
     *
     * @return integer
     */
    public function getSalesRepUser(): int
    {
        $salesRepId = self::DEFAULT_SALESREP_ID;
        $collection = $this->salesRepCollectionFactory->create();
        $collection->addFieldToSelect('user_id')->addFieldToFilter('sales_order_entity_id', $this->getOrderId())->load();
        $collection->getColumnValues('user_id');
        $data = $collection->getData();
        if ($data) {
            $salesRepId = $data[0]['user_id'];
        }
        return $salesRepId;
    }

    /**
     * Retrieve sales representative user name
     * @param int|string|null $user_id
     * @return string
     */
    public function getSalesRepName($user_id): string
    {
        $adminUsers = $this->getAdminUsers();
        if (is_array($adminUsers) && count($adminUsers) > 0) {
            foreach ($adminUsers as $adminuser) {
                if ($adminuser['value'] == $user_id) {
                    return $adminuser['label'];
                }
            }
        }
        return self::DEFAULT_SALESREP_NAME;
    }

    /**
     * Retrieve admin users array
     *
     * @return array
     */
    public function getAdminUsers(): array
    {
        $adminUsers = [];

        foreach ($this->userCollectionFactory->create() as $user) {
            $adminUsers[] = [
                'value' => $user->getId(),
                'label' => $user->getName()
            ];
        }
        return $adminUsers;
    }
}
