<?php

namespace Pinpoint\SalesRep\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\User\Model\User;
use Pinpoint\SalesRep\Plugin\AdminAddComment;

/**
 * Class CheckSalesRepAdminNewOrder
 */
class CheckSalesRepAdminNewOrder implements ObserverInterface
{
    /**
     * @var AdminAddComment
     */
    protected AdminAddComment $addComment;

    /**
     * @var  \Magento\User\Model\User
     */
    protected User $currentUser;

    /**
     * Constructor
     * @param AdminAddComment $addComment
     */
    public function __construct(AdminAddComment $addComment)
    {
        $this->addComment = $addComment;
    }

    /**
     * Execute observer
     * @inheritDoc
     */
    public function execute(Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();

        // Get Order Id
        $order_id = $order->getId();

        // Check if this order has a sales representative
        $salesRepId = $this->addComment->checkSalesRep($order_id);
        if (!$salesRepId) {

            // Does not have sales representative, lets assign then send an email
            $this->currentUser = $this->addComment->getCurrentAdminUser();
            if (is_object($this->currentUser)) {
                $this->addComment->assignSalesRep($order_id, $this->currentUser->getId());
            }
        }
        return $this;
    }
}
