<?php

namespace Pinpoint\SalesRep\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\User\Model\User;
use Pinpoint\SalesRep\Plugin\AdminAddComment;

/**
 * Class CheckSalesRepShipment
 */
class CheckSalesRepShipment implements ObserverInterface
{
    /**
     * @var AdminAddComment
     */
    protected AdminAddComment $addComment;

    /**
     * @var  \Magento\User\Model\User
     */
    protected User $currentUser;

    /**
     * Constructor
     * @param AdminAddComment $addComment
     */
    public function __construct(AdminAddComment $addComment)
    {
        $this->addComment = $addComment;
    }

    /**
     * Execute observer
     * @inheritDoc
     */
    public function execute(Observer $observer)
    {
        $shipment = $observer->getEvent()->getShipment();
        $order = $shipment->getOrder();

        // Get Order Id
        $order_id = $order->getId();

        // Check if this order has a sales representative
        $salesRepId = $this->addComment->checkSalesRep($order_id);
        if (!$salesRepId) {

            // Does not have sales representative, lets assign then send an email
            $this->currentUser = $this->addComment->getCurrentAdminUser();
            $this->addComment->assignSalesRep($order_id, $this->currentUser->getId());

        } else {

            // A status change detected, send email to the current salesrep
            $comment = $shipment->getCommentsCollection()->getFirstItem()->getComment();
            $this->addComment->sendEmailToSalesRep($salesRepId, $order, 'Order is Shipped' . ($comment ? ' / ' . $comment : ''));
        }

        return $this;
    }
}
